/*
 * Manager for various I/O types.
 * (C) 2004 Fujita Tomonori <tomof@acm.org>
 * This code is licenced under the GPL.
 */

#include <linux/module.h>
#include <linux/list.h>
#include <linux/spinlock.h>
#include <linux/errno.h>

#include "iscsi.h"
#include "target.h"
#include "target_device.h"
#include "iotype.h"

static LIST_HEAD(iotypes);
static rwlock_t iotypes_lock = RW_LOCK_UNLOCKED;

static struct iotype *find_iotype(const char *name)
{
	struct iotype *iot = NULL;

	list_for_each_entry(iot, &iotypes, iot_list) {
		if (strcmp(iot->name, name) == 0)
			return iot;
	}
	return NULL;
}

struct iotype *get_iotype(const char *name)
{
	struct iotype *iot;
	read_lock(&iotypes_lock);

	iot = find_iotype(name);
	if (iot && !try_inc_mod_count(iot->owner))
		iot = NULL;
	read_unlock(&iotypes_lock);

	return iot;
}

void put_iotype(struct iotype *iot)
{
	__MOD_DEC_USE_COUNT(iot->owner);
}

int register_iotype(struct iotype *iot)
{
	int res = 0;
	struct iotype *p;

	write_lock(&iotypes_lock);

	p = find_iotype(iot->name);

	if (p)
		res = -EBUSY;
	else
		list_add_tail(&iot->iot_list, &iotypes);

	write_unlock(&iotypes_lock);

	return res;
}

int unregister_iotype(struct iotype *iot)
{
	int res = 0;
	struct iotype *p;

	write_lock(&iotypes_lock);

	p = find_iotype(iot->name);

	if (p && p == iot)
		list_del_init(&iot->iot_list);
	else
		res = -EINVAL;

	write_unlock(&iotypes_lock);


	return res;
}
